<?php
defined('BASEPATH') OR exit('No direct script access allowed');
require_once (APPPATH . 'third_party/pkg/jwt/autoload.php');
use pkg\jwt\src\JWT;
use pkg\jwt\src\ExpiredException;

/**
 * JWT类
 * 引用pkg-jwt类库，提供给CI框架调用
 *
 * @author wyq & lyj
 * @date 2019/09/23
 */
class SF_JWT {

    /**
     * 加密密钥数组
     */
    protected $keys;
    /**
     * CI实例
     */
    protected $CI;
    /**
     * token生成时间
     */
    protected $iat;
    /**
     * token失效时间
     */
    protected $exp;
    /**
     * jwt payload
     */
    protected $payload;
    /**
     * jwt msg
     */
    protected $msg;

    /**
     * 参数初始化
     */
    public function __construct()
    {
        $this->CI = &get_instance();
        $this->CI->load->config('jwt', true);
        $config = config_item('jwt');
        $this->keys = $config['keys'];
        $this->iat = $_SERVER['REQUEST_TIME'];
        $this->exp = $_SERVER['REQUEST_TIME'] + $config['timeout'];

        JWT::$alg = $config['alg'];
        JWT::$timestamp = $_SERVER['REQUEST_TIME'];
    }


    /**
     * 生成token值
     *
     * @param $payload array 消息体
     * @return string 加密的token值
     */
    public function encode(array $payload)
    {
        try
        {
            //生成时间
            if (!isset($payload['iat']))
            {
                $payload['iat'] = $this->iat;
            }
            //失效时间
            if (!isset($payload['exp']))
            {
                $payload['exp'] = $this->exp;
            }
            //CSRF token
            if (!isset($payload['cst']))
            {
                $payload['cst'] = random_string('md5');
            }
            //产生token
            $msg = JWT::encode($payload, $this->keys);
            //保存payload
            $this->payload = $payload;
            //保存msg
            $this->msg = $msg;
            return $msg;
        }
        catch (Exception $e)
        {
            $this->_log_message('error', $e->getMessage());
            return false;
        }
    }


    /**
     * 校验token值
     *
     * @return object|false 消息体|失败返回false
     */
    public function decode($token)
    {
        try
        {
            JWT::decode($token, $this->keys, ['RS256'], $payload);
            //保存payload
            $this->payload = $payload;
            return true;
        }
        catch (ExpiredException $e)
        {
            $this->_log_message('error', $e->getMessage());
            return false;
        }
    }


    /**
     * 获取jwt payload
     *
     * @return string
     */
    public function get_payload()
    {
        if (!$this->payload)
        {
            $this->payload = '';
        }
        return $this->payload;
    }


    /**
     * 获取jwt msg
     *
     * @return string
     */
    public function get_msg()
    {
        if (!$this->msg)
        {
            $this->msg = '';
        }
        return $this->msg;
    }


    /**
     * 安全检查
     */
    protected function _check()
    {
        $this->_check_ajax();
        $this->_check_referer();
        $this->_check_csrf();
    }


    /**
     * 校验是否ajax请求
     */
    protected function _check_ajax()
    {
        if (!$this->CI->input->is_ajax_request())
        {
            $this->_log_message('error', 'is not ajax request.');
            set_status_header('404');
            die;
        }
    }


    /**
     * 校验referer
     */
    protected function _check_referer()
    {
        if (!isset($_SERVER['HTTP_REFERER']))
        {
            $this->_log_message('error', 'no referer.');
            set_status_header('404');
            die;
        }
        if (strpos($_SERVER['HTTP_REFERER'], $_SERVER['HTTP_ORIGIN']) !== 0)
        {
            $this->_log_message('error', 'invalid referer.' . $_SERVER['HTTP_REFERER']);
            set_status_header('404');
            die;
        }
    }


    /**
     * CSRF检查
     */
    protected function _check_csrf()
    {
        if (!isset($_SERVER['HTTP_CSRF_TOKEN']) || !hash_equals($_SERVER['HTTP_CSRF_TOKEN'], $this->payload['cst']))
        {
            $this->_log_message('error', 'invalid CSRF token.');
            set_status_header('404');
            die;
        }
    }


    /**
     * 记录日志
     *
     * @param $level
     * @param $msg
     */
    protected function _log_message($level, $msg)
    {
        $msg = 'msg: ' . $msg . ', IP: ' . $this->CI->input->ip_address;
        log_message_r($level, $msg);
    }

}
